@@ -154,8 +154,8 @@ drake_debug <- function(
       assign(dep, value, envir = config$envir_targets, inherits = FALSE)
     }
   }
-  config$layout[[target]]$command_build <- cdl_preprocess_command(
-    debug_command(config$layout[[target]]$command)
+  config$spec[[target]]$command_build <- cds_preprocess_command(
+    debug_command(config$spec[[target]]$command)
   )
   config$ht_dynamic <- ht_new()
   meta <- drake_meta_(target = target, config = config)
@@ -172,7 +172,7 @@ debug_command <- function(command) {
     debug_command_char(command)
   } else {
     . <- NULL
-    out <- safe_deparse(command)
+    out <- safe_deparse(command, backtick = TRUE)
     out <- debug_command_char(out)
     parse(text = out)[[1]]
   }
