@@ -13,14 +13,14 @@
 #'   w = c("a", "a", "b", "b"),
 #'   x = seq_len(4),
 #'   y = target(x + 1, dynamic = map(x)),
-#'   z = target(list(y = y, w = w), dynamic = group(y, .by = w))
+#'   z = target(sum(x) + sum(y), dynamic = group(x, y, .by = w))
 #' )
 #' make(plan)
 #' subtargets(y)
-#' readd(subtargets(y)[1], character_only = TRUE)
-#' readd(subtargets(y)[2], character_only = TRUE)
-#' readd(subtargets(z)[1], character_only = TRUE)
-#' readd(subtargets(z)[2], character_only = TRUE)
+#' subtargets(z)
+#' readd(x)
+#' readd(y)
+#' readd(z)
 #' })
 #' }
 subtargets <- function(
@@ -41,6 +41,7 @@ subtargets <- function(
 }
 
 #' @title Read a trace of a dynamic target.
+#' \lifecycle{experimental}
 #' @export
 #' @seealso [get_trace()], [subtargets()]
 #' @description Read a target's dynamic trace from the cache.
@@ -55,9 +56,9 @@ subtargets <- function(
 #' @param trace Character, name of the trace
 #'   you want to extract. Such trace names are declared
 #'   in the `.trace` argument of `map()`, `cross()` or `group()`.
-#' @param trace Character, name of a target from which to extract
-#'   a trace.
-#' @param target The name of a dynamic target with one or more traces
+#' @param target Symbol or character,
+#'   depending on the value of `character_only`.
+#'   `target` is T=the name of a dynamic target with one or more traces
 #'   defined using the `.trace` argument of dynamic `map()`, `cross()`,
 #'   or `group()`.
 #' @examples
@@ -69,15 +70,18 @@ subtargets <- function(
 #'
 #'   # The first trace lets us see the values of w
 #'   # that go with the sub-targets of y.
-#'   y = target(c(w, x), dynamic = cross(w, x, .trace = w)),
+#'   y = target(paste0(w, x), dynamic = cross(w, x, .trace = w)),
 #'
 #'   # We can use the trace as a grouping variable for the next
 #'   # group().
-#'   w_tr = get_trace("w", y),
+#'   w_tr = read_trace("w", y),
 #'
 #'   # Now, we use the trace again to keep track of the
 #'   # values of w corresponding to the sub-targets of z.
-#'   z = target(y, dynamic = group(y, .by = w_tr, .trace = w_tr))
+#'   z = target(
+#'     paste0(y, collapse = "-"),
+#'     dynamic = group(y, .by = w_tr, .trace = w_tr)
+#'   )
 #' )
 #' make(plan)
 #'
@@ -85,76 +89,37 @@ subtargets <- function(
 #' # That way, we know which values of `w` correspond
 #' # to the sub-targets of `y`.
 #' readd(y)
-#' read_trace("w", "y")
+#' read_trace("w", y)
 #'
 #' # And we know which values of `w_tr` (and thus `w`)
-#' # match up with the sub-targets of `z`.
+#' # match up with the sub-targets of `y`.
 #' readd(z)
-#' read_trace("w_tr", "z")
+#' read_trace("w_tr", z)
 #' })
 #' }
 read_trace <- function(
   trace,
   target,
   cache = drake::drake_cache(path = path),
-  path = NULL
+  path = NULL,
+  character_only = FALSE
 ) {
   if (is.null(cache)) {
     stop("cannot find drake cache.")
   }
   cache <- decorate_storr(cache)
+  if (!character_only) {
+    target <- as.character(substitute(target))
+  }
   value <- cache$get(standardize_key(target), use_cache = FALSE)
-  get_trace(trace, value)
-}
-
-#' @title Get a trace of a dynamic target's value.
-#' @export
-#' @seealso [read_trace()], [subtargets()]
-#' @description Get the dynamic trace of a target's value.
-#'   Best used inside a `drake` plan.
-#' @details In dynamic branching, the trace keeps track
-#'   of how the sub-targets were generated.
-#'   It reminds us the values of grouping variables
-#'   that go with individual sub-targets.
-#' @return The dynamic trace of one target in another:
-#'   a vector of values from a grouping variable.
-#' @param trace Character, name of the trace
-#'   you want to extract. Such trace names are declared
-#'   in the `.trace` argument of `map()`, `cross()` or `group()`.
-#' @param value The return value of the target with the trace.
-#' @examples
-#' \dontrun{
-#' isolate_example("demonstrate dynamic trace", {
-#' plan <- drake_plan(
-#'   w = LETTERS[seq_len(3)],
-#'   x = letters[seq_len(2)],
-#'
-#'   # The first trace lets us see the values of w
-#'   # that go with the sub-targets of y.
-#'   y = target(c(w, x), dynamic = cross(w, x, .trace = w)),
-#'
-#'   # We can use the trace as a grouping variable for the next
-#'   # group().
-#'   w_tr = get_trace("w", y),
-#'
-#'   # Now, we use the trace again to keep track of the
-#'   # values of w corresponding to the sub-targets of z.
-#'   z = target(y, dynamic = group(y, .by = w_tr, .trace = w_tr))
-#' )
-#' make(plan)
-#'
-#' # We can read the trace outside make().
-#' read_trace("w", "y")
-#' read_trace("w_tr", "z")
-#' })
-#' }
-get_trace <- function(trace, value) {
   attr(value, "dynamic_trace")[[trace]]
 }
 
 dynamic_build <- function(target, meta, config) {
-  subtargets <- config$layout[[target]]$subtargets
-  meta$time_command <- proc.time() - meta$time_start
+  subtargets <- config$spec[[target]]$subtargets
+  if (config$log_build_times) {
+    meta$time_command <- proc_time() - meta$time_start
+  }
   value <- config$cache$mget_hash(subtargets)
   value <- append_trace(target, value, config)
   class(value) <- "drake_dynamic"
@@ -162,12 +127,12 @@ dynamic_build <- function(target, meta, config) {
 }
 
 append_trace <- function(target, value, config) {
-  layout <- config$layout[[target]]
-  if (!length(layout$deps_dynamic_trace)) {
+  spec <- config$spec[[target]]
+  if (!length(spec$deps_dynamic_trace)) {
     return(value)
   }
-  dynamic <- layout$dynamic
-  trace <- get_trace_impl(dynamic, value, layout, config)
+  dynamic <- spec$dynamic
+  trace <- get_trace_impl(dynamic, value, spec, config)
   trace <- subset_trace(trace, config)
   attr(value, "dynamic_trace") <- trace
   value
@@ -182,47 +147,47 @@ subset_trace <- function(trace, config) {
   })
 }
 
-get_trace_impl <- function(dynamic, value, layout, config) {
+get_trace_impl <- function(dynamic, value, spec, config) {
   UseMethod("get_trace_impl")
 }
 
-get_trace_impl.map <- function(dynamic, value, layout, config) {
+get_trace_impl.map <- function(dynamic, value, spec, config) {
   trace <- lapply(
-    layout$deps_dynamic_trace,
+    spec$deps_dynamic_trace,
     get,
     envir = config$envir_targets,
     inherits = FALSE
   )
   trace <- lapply(trace, chr_dynamic)
-  names(trace) <- layout$deps_dynamic_trace
+  names(trace) <- spec$deps_dynamic_trace
   trace
 }
 
-get_trace_impl.cross <- function(dynamic, value, layout, config) {
+get_trace_impl.cross <- function(dynamic, value, spec, config) {
   size <- lapply(
-    layout$deps_dynamic,
+    spec$deps_dynamic,
     get_dynamic_size,
     config = config
   )
   index <- lapply(size, seq_len)
-  names(index) <- layout$deps_dynamic
+  names(index) <- spec$deps_dynamic
   index <- rev(expand.grid(rev(index)))
   trace <- lapply(
-    layout$deps_dynamic_trace,
+    spec$deps_dynamic_trace,
     get,
     envir = config$envir_targets,
     inherits = FALSE
   )
   trace <- lapply(trace, chr_dynamic)
-  names(trace) <- layout$deps_dynamic_trace
-  trace <- lapply(layout$deps_dynamic_trace, function(key) {
+  names(trace) <- spec$deps_dynamic_trace
+  trace <- lapply(spec$deps_dynamic_trace, function(key) {
     trace[[key]][index[[key]]]
   })
-  names(trace) <- layout$deps_dynamic_trace
+  names(trace) <- spec$deps_dynamic_trace
   trace
 }
 
-get_trace_impl.group <- function(dynamic, value, layout, config) {
+get_trace_impl.group <- function(dynamic, value, spec, config) {
   by_key <- which_by(dynamic)
   by_value <- get(by_key, envir = config$envir_targets, inherits = FALSE)
   trace <- list(unique(by_value))
@@ -255,10 +220,12 @@ register_subtargets <- function(target, static_ok, dynamic_ok, config) {
   }
   if (length(subtargets_all)) {
     register_in_graph(target, subtargets_all, config)
-    register_in_layout(target, subtargets_all, config)
+    register_in_spec(target, subtargets_all, config)
   }
   ndeps <- length(subtargets_build)
   if (ndeps) {
+    config$logger$minor("register", ndeps, "subtargets", target = target)
+    ht_set(config$ht_is_subtarget, subtargets_build)
     register_in_loop(subtargets_build, config)
     register_in_queue(subtargets_build, 0, config)
     register_in_counter(subtargets_build, config)
@@ -307,31 +274,44 @@ register_in_graph <- function(target, subtargets, config) {
   )
 }
 
-register_in_layout <- function(target, subtargets, config) {
+register_in_spec <- function(target, subtargets, config) {
+  spec <- config$spec[[target]]
+  dynamic <- spec$dynamic
   lapply(
     seq_along(subtargets),
-    register_subtarget_layout,
+    register_subtarget_spec,
     parent = target,
     subtargets = subtargets,
+    spec = spec,
+    dynamic = dynamic,
     config = config
   )
-  config$layout[[target]]$subtargets <- subtargets
+  config$spec[[target]]$subtargets <- subtargets
 }
 
-register_subtarget_layout <- function(index, parent, subtargets, config) {
+register_subtarget_spec <- function(
+  index,
+  parent,
+  subtargets,
+  dynamic,
+  spec,
+  config
+) {
   subtarget <- subtargets[index]
-  layout <- config$layout[[parent]]
-  layout$target <- subtarget
-  layout$subtarget_index <- index
-  layout$subtarget_parent <- parent
-  layout$is_dynamic <- FALSE
-  layout$is_subtarget <- TRUE
-  mem_deps <- all.vars(layout$dynamic)
-  layout$dynamic <- NULL
-  layout$deps_build$memory <- unique(c(layout$deps_build$memory, mem_deps))
-  layout$seed <- seed_from_basic_types(config$seed, layout$seed, subtarget)
-  layout <- register_dynamic_subdeps(layout, index, parent, config)
-  config$layout[[subtarget]] <- layout
+  spec$target <- subtarget
+  spec$subtarget_index <- index
+  spec$subtarget_parent <- parent
+  mem_deps <- all.vars(spec$dynamic)
+  spec$dynamic <- NULL
+  spec$deps_build$memory <- unique(c(spec$deps_build$memory, mem_deps))
+  spec$seed <- seed_from_basic_types(config$seed, spec$seed, subtarget)
+  spec <- register_dynamic_subdeps(dynamic, spec, index, parent, config)
+  assign(
+    x = subtarget,
+    value = spec,
+    envir = config$spec,
+    inherits = FALSE
+  )
 }
 
 register_in_loop <- function(targets, config) {
@@ -362,20 +342,20 @@ dynamic_pad_revdep_keys <- function(target, config) {
   increase_revdep_keys(config$queue, target, config)
 }
 
-register_dynamic_subdeps <- function(layout, index, parent, config) {
-  index_deps <- subtarget_deps(parent, index, config)
-  for (dep in layout$deps_dynamic) {
+register_dynamic_subdeps <- function(dynamic, spec, index, parent, config) {
+  index_deps <- subtarget_deps(dynamic, parent, index, config)
+  for (dep in spec$deps_dynamic) {
     if (is_dynamic(dep, config)) {
-      subdeps <- config$layout[[dep]]$subtargets[index_deps[[dep]]]
-      layout$deps_build$memory <- c(layout$deps_build$memory, subdeps)
-      layout$deps_build$memory <- setdiff(layout$deps_build$memory, dep)
+      subdeps <- config$spec[[dep]]$subtargets[index_deps[[dep]]]
+      spec$deps_build$memory <- c(spec$deps_build$memory, subdeps)
+      spec$deps_build$memory <- setdiff(spec$deps_build$memory, dep)
     }
   }
-  layout
+  spec
 }
 
 is_dynamic <- function(target, config) {
-  config$layout[[target]]$is_dynamic %||% FALSE
+  ht_exists(config$ht_is_dynamic, target)
 }
 
 is_dynamic_dep <- function(target, config) {
@@ -383,7 +363,7 @@ is_dynamic_dep <- function(target, config) {
 }
 
 is_subtarget <- function(target, config) {
-  config$layout[[target]]$is_subtarget %||% FALSE
+  ht_exists(config$ht_is_subtarget, target)
 }
 
 as_dynamic <- function(x) {
@@ -395,36 +375,7 @@ as_dynamic <- function(x) {
 }
 
 dynamic_subvalue <- function(value, index) {
-  UseMethod("dynamic_subvalue")
-}
-
-dynamic_subvalue.data.frame <- function(value, index) {
-  value[index,, drop = FALSE] # nolint
-}
-
-dynamic_subvalue.drake_dynamic <- function(value, index) {
-  dynamic_subvalue_vector(value, index)
-}
-
-dynamic_subvalue.default <- function(value, index) {
-  if (is.null(dim(value))) {
-    dynamic_subvalue_vector(value, index)
-  } else {
-    dynamic_subvalue_array(value, index)
-  }
-}
-
-dynamic_subvalue_array <- function(value, index) {
-  ref <- slice.index(value, 1L)
-  out <- value[ref %in% index]
-  dim <- dim(value)
-  dim[1] <- length(index)
-  dim(out) <- dim
-  out
-}
-
-dynamic_subvalue_vector <- function(value, index) {
-  value[index]
+  vec_slice(x = value, i = index)
 }
 
 match_dynamic_call <- function(dynamic) {
@@ -468,7 +419,7 @@ def_group <- function(..., .by = NULL, .trace = NULL) {
 # nocov end
 
 subtarget_names <- function(target, config) {
-  dynamic <- config$layout[[target]]$dynamic
+  dynamic <- config$spec[[target]]$dynamic
   hashes <- dynamic_hash_list(dynamic, target, config)
   hashes <- subtarget_hashes(dynamic, target, hashes, config)
   hashes <- vapply(hashes, shorten_dynamic_hash, FUN.VALUE = character(1))
@@ -486,7 +437,7 @@ max_expand_dynamic <- function(targets, config) {
 }
 
 shorten_dynamic_hash <- function(hash) {
-  digest::digest(hash, algo = "murmur32", serialize = FALSE)
+  digest_murmur32(hash, serialize = FALSE)
 }
 
 dynamic_hash_list <- function(dynamic, target, config) {
@@ -494,14 +445,14 @@ dynamic_hash_list <- function(dynamic, target, config) {
 }
 
 dynamic_hash_list.map <- function(dynamic, target, config) {
-  deps <- sort(config$layout[[target]]$deps_dynamic)
+  deps <- sort(config$spec[[target]]$deps_dynamic)
   hashes <- lapply(deps, read_dynamic_hashes, config = config)
   assert_equal_branches(target, deps, hashes)
   hashes
 }
 
 dynamic_hash_list.cross <- function(dynamic, target, config) {
-  deps <- sort(config$layout[[target]]$deps_dynamic)
+  deps <- sort(config$spec[[target]]$deps_dynamic)
   lapply(deps, read_dynamic_hashes, config = config)
 }
 
@@ -509,7 +460,12 @@ dynamic_hash_list.group <- function(dynamic, target, config) {
   deps <- sort(which_vars(dynamic))
   out <- lapply(deps, read_dynamic_hashes, config = config)
   if (!is.null(dynamic$.by)) {
-    out[["_by"]] <- read_dynamic_hashes(deparse(dynamic$.by), config)
+    key <- direct_deparse(
+      dynamic$.by,
+      control = deparse_control_default,
+      backtick = FALSE
+    )
+    out[["_by"]] <- read_dynamic_hashes(key, config)
   }
   assert_equal_branches(target, which_vars(dynamic), out)
   out
@@ -538,7 +494,7 @@ read_dynamic_hashes <- function(target, config) {
   meta <- config$cache$get(target, namespace = "meta")
   if (is.null(meta$dynamic_hashes)) {
     value <- config$cache$get(target)
-    meta$dynamic_hashes <- dynamic_hashes(value, meta$size, config)
+    meta$dynamic_hashes <- dynamic_hashes(value, meta$size_vec, config)
   }
   meta$dynamic_hashes
 }
@@ -567,23 +523,18 @@ subtarget_hashes.group <- function(dynamic, target, hashes, config) {
   unname(tapply(hashes, INDEX = by, FUN = paste, collapse = " "))
 }
 
-subtarget_deps <- function(target, index, config) {
-  dynamic <- config$layout[[target]]$dynamic
-  subtarget_deps_impl(dynamic, target, index, config)
+subtarget_deps <- function(dynamic, target, index, config) {
+  UseMethod("subtarget_deps")
 }
 
-subtarget_deps_impl <- function(dynamic, target, index, config) {
-  UseMethod("subtarget_deps_impl")
-}
-
-subtarget_deps_impl.map <- function(dynamic, target, index, config) {
+subtarget_deps.map <- function(dynamic, target, index, config) {
   vars <- all.vars(dynamic)
   out <- as.list(rep(as.integer(index), length(vars)))
   names(out) <- vars
   out
 }
 
-subtarget_deps_impl.cross <- function(dynamic, target, index, config) {
+subtarget_deps.cross <- function(dynamic, target, index, config) {
   vars <- all.vars(dynamic)
   size <- unlist(lapply(vars, get_dynamic_size, config = config))
   out <- grid_index(index, size)
@@ -592,7 +543,7 @@ subtarget_deps_impl.cross <- function(dynamic, target, index, config) {
   out
 }
 
-subtarget_deps_impl.group <- function(
+subtarget_deps.group <- function(
   dynamic,
   target,
   index,
@@ -614,7 +565,7 @@ get_dynamic_size <- function(target, config) {
   if (ht_exists(config$ht_dynamic_size, target)) {
     return(ht_get(config$ht_dynamic_size, target))
   }
-  size <- config$cache$get(target, namespace = "meta")$size
+  size <- config$cache$get(target, namespace = "meta")$size_vec
   stopifnot(size > 0L)
   ht_set(config$ht_dynamic_size, x = target, value = size)
   size
@@ -647,5 +598,9 @@ which_vars <- function(dynamic) {
 }
 
 which_by <- function(dynamic) {
-  deparse(dynamic$.by)
+  direct_deparse(
+    dynamic$.by,
+    control = deparse_control_default,
+    backtick = FALSE
+  )
 }
