@@ -55,8 +55,12 @@ try_build <- function(target, meta, config) {
     return(dynamic_build(target, meta, config))
   }
   retries <- 0L
-  layout <- config$layout[[target]] %||% list()
-  max_retries <- as.numeric(layout$retries %||NA% config$retries)
+  spec <- config$spec[[target]] %|||% list()
+  max_retries <- spec$retries
+  if (is.null(max_retries) || is.na(max_retries)) {
+    max_retries <- config$retries
+  }
+  max_retries <- as.integer(max_retries)
   while (retries <= max_retries) {
     if (retries > 0L) {
       config$logger$major(
@@ -99,11 +103,15 @@ with_seed_timeout <- function(target, meta, config) {
 }
 
 resolve_timeouts <- function(target, config) {
-  layout <- config$layout[[target]] %||% list()
+  spec <- config$spec[[target]] %|||% list()
   vapply(
     X = c("cpu", "elapsed"),
     FUN = function(key) {
-      layout[[key]] %||NA% config[[key]]
+      out <- spec[[key]]
+      if (is.null(out) || is.na(out)) {
+        out <- config[[key]]
+      }
+      out
     },
     FUN.VALUE = numeric(1)
   )
@@ -169,7 +177,9 @@ get_seed <- function() {
 # https://github.com/arendsee/rmonad/blob/14bf2ef95c81be5307e295e8458ef8fb2b074dee/R/to-monad.R#L68 # nolint
 with_handling <- function(target, meta, config) {
   warnings <- messages <- NULL
-  start <- proc.time()
+  if (config$log_build_times) {
+    start <- proc_time()
+  }
   withCallingHandlers(
     value <- with_call_stack(target = target, config = config),
     warning = function(w) {
@@ -184,7 +194,9 @@ with_handling <- function(target, meta, config) {
       invokeRestart("muffleMessage")
     }
   )
-  meta$time_command <- proc.time() - start
+  if (config$log_build_times) {
+    meta$time_command <- proc_time() - start
+  }
   meta$warnings <- prepend_fork_advice(warnings)
   meta$messages <- messages
   if (inherits(value, "error")) {
@@ -232,7 +244,7 @@ with_call_stack <- function(target, config) {
     e$calls <- head(sys.calls()[-seq_len(frame + 7)], -2)
     signalCondition(e)
   }
-  expr <- config$layout[[target]]$command_build
+  expr <- config$spec[[target]]$command_build
   # Need to make sure the environment is locked for running commands.
   # Why not just do this once at the beginning of `make()`?
   # Because do_prework() and future::value()
@@ -319,7 +331,7 @@ conclude_build <- function(build, config) {
   value <- assign_format(
     target = target,
     value = value,
-    format = config$layout[[target]]$format,
+    format = config$spec[[target]]$format,
     config = config
   )
   store_outputs(target = target, value = value, meta = meta, config = config)
@@ -354,7 +366,7 @@ sanitize_format.drake_format_fst <- function(x, target, config) { # nolint
     msg <- paste0(
       "You selected fst format for target ", target,
       ", so drake will convert it from class ",
-      safe_deparse(class(x$value)),
+      safe_deparse(class(x$value), backtick = TRUE),
       " to a plain data frame."
     )
     warning(msg, call. = FALSE)
@@ -370,7 +382,7 @@ sanitize_format.drake_format_fst_dt <- function(x, target, config) { # nolint
     msg <- paste0(
       "You selected fst_dt format for target ", target,
       ", so drake will convert it from class ",
-      safe_deparse(class(x$value)),
+      safe_deparse(class(x$value), backtick = TRUE),
       " to a data.table object."
     )
     warning(msg, call. = FALSE)
@@ -386,7 +398,7 @@ sanitize_format.drake_format_diskframe <- function(x, target, config) { # nolint
     msg <- paste0(
       "You selected disk.frame format for target ", target,
       ", so drake will try to convert it from class ",
-      safe_deparse(class(x$value)),
+      safe_deparse(class(x$value), backtick = TRUE),
       " to a disk.frame object. For optimal performance ",
       "please create disk.frame objects yourself using an outdir ",
       "on the same drive as drake's cache ",
@@ -406,8 +418,10 @@ assign_to_envir <- function(target, value, config) {
   if (is_subtarget(target, config)) {
     return()
   }
-  memory_strategy <- config$layout[[target]]$memory_strategy %||NA%
-    config$memory_strategy
+  memory_strategy <- config$spec[[target]]$memory_strategy
+  if (is.null(memory_strategy) || is.na(memory_strategy)) {
+    memory_strategy <- config$memory_strategy
+  }
   skip_memory <- memory_strategy %in% c("autoclean", "unload", "none")
   if (skip_memory) {
     return()
@@ -443,7 +457,7 @@ value_format.default <- function(value, target, config) {
 }
 
 assert_output_files <- function(target, meta, config) {
-  deps <- config$layout[[target]]$deps_build
+  deps <- config$spec[[target]]$deps_build
   if (!length(deps$file_out)) {
     return()
   }
@@ -531,7 +545,7 @@ store_failure <- function(target, meta, config) {
 set_progress <- function(target, value, config) {
   skip_progress <- !identical(config$running_make, TRUE) ||
     !config$log_progress ||
-    (config$layout[[target]]$imported %||% FALSE)
+    (config$spec[[target]]$imported %||% FALSE)
   if (skip_progress) {
     return()
   }
